*--------------------------------------------------------------------------
* GRAPHICAL RESULTS: to consider more reforms, just add an extra row to sims.dta
*-------------------------------------------------------------------------

/******************************************
Graphical Results:

The code uses the taxcost-program: see this is you want details on how it works.

The graphic results require input file sims.dta, which has information
on which types of taxpayers and which tax reforms should be analyzed.
To analyze another tax change/taxpayer type combination, just fill in
all the variables (except for "source," which is not necessary) in the
sims.dta file and run the code again.  Open sims.dta and enter the command
"describe" to see what all the variables mean.  The "zero" variable allows 
the calculation of the benefits of reoptimizing to the post-year tax code
relative to optimizizing to 0 taxes.

The output of the graphical results of the form `prefix'-all-`ma'.gph, 
where `prefix' is an abbreviation input into the sims.dta file
for the tax change and group of individuals under consideration and `ma'
refers to the margin ("Extensive" or "Intensive").  These results combine
several graphs, including the change in tax rates (`prefix'-taxrate-`ma'.gph), the financial benefit
of reoptimizing (`prefix'-financialcost-`ma'.gph), the percentage benefit of reoptimizing (`prefix'-percent-`ma'.gph), and the change
in income from reoptimizing (`prefix'-income-`ma'.gph).  This is created in Stata
graph format.

Notes:

(1) The eitc graphs are edited manually for low incomes (<$8K) to fix the rounding off error in this region; the true values for low incomes is
easily calculated numerically and that is what we use to correct for the round off error.  

Source of median income:
http://www.census.gov/hhes/www/income/histinc/f07ar.html
http://www.census.gov/hhes/www/income/histinc/h05.html
Source of CPI for all urban consumers:
http://data.bls.gov/cgi-bin/surveymost?blsf
Pay of accountants and tax preparers:
http://www.bls.gov/oco/ocos001.htm#earnings

Source of 99.5th percentile (Saez's website):
http://elsa.berkeley.edu/~saez/TabFig2006.xls

Source of lower income percentiles (from CPS data):
See "income percentiles.pdf" in the main directory
Similar data is available at: http://www.census.gov/hhes/www/income/histinc/h01ar.html

Last updated: February 8th, 2011 by Jessica Laird.  Can be contacted at jalpernl@gmail.com
***************/

clear
set mem 1000m
set more off

capture log close
log using graphs.log, replace

*SET THE GRID TO BE USED IN BOTH taxcost AND THE EXTRA TAXSCHEDULE
local grid = 100

*Use data on tax reforms
use sims, clear

*Specify elasticity you want to use e=elast.
*Note: to change elast. search for g, which is used later in the code
replace e=.5
gen g=1/e

*Rename variables to the necessary names for taxcost
ren preyear year
ren maritalstat mstat
ren exemptions depx
ren partnerwag swages 
ren numdepend depchild

*code will identify each original observation after the multiples are added
gen code=_n

*Since you need to put preyear=0 to get zero-calculations
replace year=0 if zero==1
drop zero

*Give the zero calcs the same characteristics as the others
replace mstat = mstat[1] in 3/4
replace depx = depx[1] in 3/4
replace swages  = swages[1] in 3/4
replace depchild = depchild[1] in 3/4

local numsims = _N

*Load incomes to use as base for multiples
forvalues y=1/`numsims'{
	preserve
	use sims, clear
	keep if _n== `y'
	local preyear=preyear
	* Pull in data from income dataset			
	use median-cpi, clear
	* Pick the median income from the relevant year of "all families"
	* This number does not matter much, since it is just the base for finding multiples
	keep if year==`preyear'
	local medianincome`y' = allfamilies
	restore
}

*Load the right medianincome to each row
gen pwages=0
forvalues y=1/`numsims'{
	replace pwages=`medianincome`y'' in `y'
}

save inputgraphs, replace

*Specify the mulitples you want to use (each multiple will be a "observation" 
*After the loop is finished, the macro multiples will contain ALL the different, from 0.04 to 7, at steps of 0.04
*Change the stepsize, and/or topnumber if you want a smaller/larger range of incomes.
local cnt=0
forvalues i=0(0.004)7{
	local mu = `i'
	if `i'==0.004 local multiples = "`mu'"
	else local multiples "`multiples' `mu'"
	local cnt=`cnt'+1
}

*Expand dataset to get a row for each income
expand `cnt'

*create the "multiples" variable, and give each row the right multiple
sort code
local j=0
gen multiples=0
foreach mu of local multiples{
	local j=`j'+1
	forvalues i=1/`numsims'{
		local row = `j' + (`i'-1)*`cnt'
		capture replace multiples = `mu' in `row'
	}
}
*Replace wages by wages*multiples
capture replace pwages = pwages*multiples

*Use taxcost;
taxcost1 `grid'

*Gen taxcostid such that we can merge the input data with the taxcost data later
gen taxcostid=_n
sort taxcostid
save inputgraphs, replace

local margin = "ext int"
foreach ma of local margin{
*--------------------------------------------------
*PART OF THE OLD SECTION "CLEANUP" LEFT OUT OF taxcost:
*---------------------------------------------------

	*Create some variables to use for the graphs (some of these are not used in the graphs, but I just keep them anyways)
	
	use taxcost_out_`ma', clear
	
	gen diffwages = pwages_reopt - pwages_noopt
	
	* Create variable for change in income required to optmize
	gen diffincome = delta_TI_`ma'
	lab var diffincome "Difference in Taxable Income: Optimizers - Non-Optimizers"		

	* Create some data on marginal tax rates
	gen mtrchange = MTR_pre - MTR_post_reopt
	gen diffMTRno =MTR_post_noopt - MTR_pre 
	gen diffNMTRno = -(MTR_post_noopt - MTR_pre)/(1-MTR_pre)*100
	gen diff = MTR_post_reopt - MTR_pre
	lab var mtrchange "Change in MTR (Pre-Post) for Optimizers"
	lab var diffMTRno "MTR Change (Post - Pre), `linetype'ed Line"
	lab var diff "MTR Change (Post - Pre) for Optimizers"

	* Create variables on average tax rates
	gen diffATRno =  ATR_post_noopt - ATR_pre
	gen diffNATRno = -(ATR_post_noopt - ATR_pre)/(1-ATR_pre)*100
	lab var diffATRno "ATR Change (Post - Pre) for Non-Optimizers"

	* Defining the hourly wage here as that which would give non-optimizers 2000 hours of work
	gen hourlywage = pwages_noopt/2000
	lab var hourlywage "Hourly Wage = Non-optimized Pre-Tax Income/2000"
	gen timevalue = hourlywage*(1-MTR_post_noopt)
	#delimit;
	lab var timevalue "Time Value = Non-Optimized Pre-Tax Hourly Wage * (1-Post-Year MTR for Non-Optimizers) 
	= Marginal Financial Value of Working in Same Job (ie $ lost from time looking for new job instead of working)";
	#delimit cr

	* Create variable for time cost of not reoptimizing, by dividing cost by 'wage'
	* This is what is used in the time cost graphs
	gen timecost = dollargain_`ma'/(timevalue)
	lab var timecost "Time Cost (=Monetary Cost/Wage)"

	*An alternative measurement
	gen pretimevalue = pwages_round*cpi/2000*(1-MTR_pre)
	lab var pretimevalue "Inflated Pre-year After-Tax Earnings for Marginal Dollar Earned (like value of leisure)"

	* Create variable for the change in the number of hours worked
	if "`ma'"=="int" gen diffhours  = diffwages/(hourlywage)
	else gen diffhours = pwages_reopt/(hourlywage)
	lab var diffhours "Difference in Hours: Optimizers - Non-Optimizers"
			
	* For labeling graphs, x axis is potential income for those on the extensive margin
	if "`ma'"=="ext" local pot "Potential"
	else local pot ""
		
	* Remove "spikes" in the MTR change by replacing MTR changes which apply to only one person
	* with the MTR changes of the individual making $grid more
	#delimit; 
	capture replace diffMTRno = diffMTRno[_n+1] if diffMTRno[_n] != diffMTRno[_n+ 1] & 
	diffMTRno[_n] != diffMTRno[_n- 1];
	#delimit cr 
	
	save taxcost_out_`ma', replace
}

*----------------------------
*Consider one reform at the time
*-----------------------------

forvalues y=1/`numsims'{

	use inputgraphs, clear
	keep if code==`y'
	
	*Some variables for the graphs
	tostring  increment, replace
	tostring topnum, generate(topnums)
	gen par1 = "("
	gen par2 = ")"
	gen zer = "0"
	egen graphrange = concat(zer par1 increment par2 topnums)
	drop par1 par2 zer
	lab var graphrange "X Axis (Income) Tick Frequency and Range for Graphs"			

	* For labeling and making graphs
	local prefix = labelpre
	local tit = title
	local not = note
	local topn = topnum
	local preperiod = year
	local postperiod = postyear 
	
	*-----------------------------
	*Creating A NEW tax schedule: this is to get more smooth tax-data to use in the graphs. 
	*we'll still use the results from taxcost to graph the utilitycost etc.
	*----------------------------
	
	use taxcost_out_int, clear

	*To get the data for the correct reform
	sort taxcostid
	merge taxcostid using inputgraphs, keep(code) 
	drop _merge
	drop if year==.
	keep if code==`y'
	
	local keepvars "year postyear mstat swages depchild depx state pwages_noopt cpi"
	keep `keepvars'
	sort pwages_noopt
	drop if pwages_noopt == .
	keep if _n==_N
	local taxrateobs = round(2*pwages_noopt/`grid')
	set obs `taxrateobs'
	gen pwages = _n*`grid'
	foreach kv of local keepvars {
		replace `kv' = `kv'[1]
	}
	
	* Put through TAXSIM for post-year
	ren year preyear
	ren postyear year
	
	* GREG FIX
	gen taxsimid=_n
	taxsim9, replace 


	* Rename variables to avoid confusion with other calculations
	drop taxsimid
	ren year postyear
	ren preyear year
	ren pwages postpwages
	ren ficar ficar_noopt
	ren frate frate_noopt
	ren fica fica_noopt
	ren fiitax fiitax_noopt 
	ren siitax siitax_noopt
	ren srate srate_noopt
	replace pwages_noopt = postpwages/1000
	
	* Put through TAXSIM for pre-year
	gen  pwages = postpwages/cpi
	if year>0{
		
		* GREG FIX
		gen taxsimid=_n
		taxsim9, replace
		drop taxsimid
	}
	if year==0{
		gen ficar=0
		gen frate=0
		gen fica=0
		gen fiitax=0
		gen siitax=0
		gen srate=0
	}

	ren year preyear
	
	*Calculate average and marginal tax rates
	gen MTR_pre = (ficar + frate + srate)/100
	gen MTR_post_noopt = (ficar_noopt + frate_noopt + srate_noopt)/100
	gen diffMTRno = MTR_post_noopt - MTR_pre 
	gen diffNMTRno = -(MTR_post_noopt - MTR_pre)/(1-MTR_pre)*100
	gen ATR_pre = (fica + fiitax + siitax)/pwages
	gen ATR_post_noopt = (fica_noopt+fiitax_noopt + siitax_noopt)/postpwages	
	gen diffATRno = ATR_post_noopt - ATR_pre
	gen diffNATRno = -(ATR_post_noopt - ATR_pre)/(1-ATR_pre)*100

	gen g = 2
		
	* Remove "spikes" in the MTR change by replacing MTR changes which apply to only one person
	* with the MTR changes of the individual making $`grid' more
	#delimit; 
	capture replace diffMTRno = diffMTRno[_n+1] if diffMTRno[_n] != diffMTRno[_n+ 1] & 
	diffMTRno[_n] != diffMTRno[_n- 1];
	#delimit cr 

	saveold output_data/`prefix'-taxsched, replace

*---------------------------------------------------------------------
* GRAPH THE DATA
*--------------------------------------------------------------------

	foreach ma of local margin{
		
		use taxcost_out_`ma', clear
		gen g = 2
		
		*To get the data for the correct reform
		sort taxcostid
		merge taxcostid using inputgraphs, keep(code) 
		drop _merge
		drop if year==.
		keep if code==`y'
		
		* Remove the logo
		graph set print logo off
			
		* Switch between average tax rate for extensive margin and marginal tr for intensive margin
		if "`ma'"=="ext" local tr "ATR"
		else local tr "MTR"
		
		* Switch between different label types for extensive and intensive margin
		if "`ma'"=="ext" local linetype = "solid"
		else local linetype = "dash"
		
		
		lab var diffN`tr'no "% Change in (1-`tr') (Right Axis)"
		lab var dollargain_`ma' "$ Gain from Reoptimizing (Left Axis)"
		lab var percentgain_`ma' "Percent Gain from Reoptimizing (Left Axis)"
		lab var diffincome "$ Change in Income from Reoptimizing (Left Axis)"
		
		*make pwages_noopt match the one from the tax schedule
		replace pwages_noopt = pwages_noopt/1000
				
		* merge datasets on tax schedule and everything else
		replace diffNMTRno = .
		replace diffNATRno = .
		append using output_data/`prefix'-taxsched
		sort pwages_noopt
		
		*Change diffN`tr'no to change in logs.  Make sure happens for graphing purposes	
		replace diffN`tr'no=log(1-`tr'_post_noopt) - log(1-`tr'_pre)

		*Edit data for presentation in graphs
		replace MTR_pre = MTR_pre * 100
		replace MTR_post_noopt = MTR_post_noopt * 100
		replace ATR_pre = ATR_pre * 100
		replace ATR_post_noopt = ATR_post_noopt * 100
		
		* Drop data above the highest income of interest for the given graph
		drop if pwages_noopt >`topn'

		* Create variables for labeling lines on graphs
		gen prmtrlab = ""
		gen MTR_post_nooptlab = ""
		gen diffMTRnolab = ""

		* For labelling graphs, when talking about the extensive margin, the graph calls income "potential income"
		if "`ma'"=="ext" local pot "Potential"
		else local pot ""
		
		* Add labels for utilty/financial cost
		gen dollargain_`ma'lab = ""
		


		* Graph how the utility/financial cost changes with income, for those with non-optimizng pre-tax income  < designated threshold
		#delimit;
		twoway
		(connected dollargain_`ma' pwages_noopt, clpat(solid) msize(vtiny) mcolor(maroon) clcolor(maroon) clwidth(thick) mlabel(dollargain_`ma'lab) mlabsize(small) mlabposition(5) mlabcolor(maroon))
		(connected diffN`tr'no pwages_noopt, yaxis(2) clpat(`linetype') msize(tiny) mcolor(ltblue) clcolor(ltblue) clwidth(medthick) msymbol(none) mlabel(diffMTRnolab) mlabsize(small) mlabposition(9) mlabcolor(ltblue))
		if g==2 & pwages_noopt < `topn' , xlabel(`grange') ylabel(#5)
		title(Financial Benefit of Reoptimizing) xtitle("", size(minuscule) margin(tiny)) ytitle("$", axis(1)) ytitle("% Change in (1-`tr')", axis(2)) graphregion(fcolor(white)) ;
		#delimit cr
		graph save util-low.gph, replace
		graph save output_figures/`prefix'-financialcost-`ma'.gph, asis replace
		save output_data/`prefix'-financialcost-`ma'.dta, replace
		
		* Add labels for time cost
		gen timecostlab = ""
		
		* Graph how financial cost as percentage of income changes with income, for those in relevant range
				
		#delimit;
		twoway 
		(connected percentgain_`ma' pwages_noopt, clpat(solid) msize(vtiny) mcolor(maroon) clcolor(maroon) clwidth(thick) mlabel(timecostlab) mlabsize(small) mlabposition(5) mlabcolor(maroon))
		 (connected diffN`tr'no pwages_noopt, yaxis(2) clpat(`linetype') msize(tiny) mcolor(ltblue) clcolor(ltblue) clwidth(medthick) msymbol(none) mlabel(diffMTRnolab) mlabsize(small) mlabposition(10) mlabcolor(ltblue))
		 if g==2 & pwages_noopt < `topn' , xlabel(`grange')
		 title("Financial Benefit of Reoptimizing: Percent of Base Year Income") xtitle("", size(minuscule) margin(tiny)) ytitle("Percent", axis(1)) ytitle("% Change in (1-`tr')", axis(2)) graphregion(fcolor(white))  ;
		#delimit cr                          
		graph save percent.gph, replace
		graph save output_figures/`prefix'-percent-`ma'.gph, asis replace    
		save output_data/`prefix'-percent-`ma'.dta, replace    
		
		******NEW GRAPH FOR ZERO2006
		if "`prefix'"=="ZER2006"{
			#delimit;
			twoway 
			(connected percentgain_`ma' pwages_noopt, clpat(solid) msize(vtiny) mcolor(maroon) clcolor(maroon) clwidth(thick) mlabel(timecostlab) mlabsize(small) mlabposition(5) mlabcolor(maroon))
			 (connected `tr'_post_noopt pwages_noopt, yaxis(2) clpat(`linetype') msize(tiny) mcolor(ltblue) clcolor(ltblue) clwidth(medthick) msymbol(none) mlabel( ) mlabsize(small) mlabposition(10) mlabcolor(ltblue))
			 if g==2 & pwages_noopt < `topn' , xlabel(`grange')
			 title("Financial Benefit of Reoptimizing: Percent of Base Year Income") xtitle("", size(minuscule) margin(tiny)) ytitle("Percent", axis(1)) ytitle("`tr'", axis(2)) graphregion(fcolor(white))  ;
			#delimit cr                          
			graph save percent.gph, replace
			graph save output_figures/`prefix'-percent-`ma'.gph, asis replace 
			save output_data/`prefix'-percent-`ma'.dta, replace  
		}	
		
		* Create special code for the EITC extensive margin graph so that the two y axes line up
		if "`prefix'" == "eitc" & "`ma'" == "Extensive" {
			* Graph how financial cost as percentage of income changes with income, for those in relevant range
			#delimit;
			twoway 
			(connected percentgain_`ma' pwages_noopt, clpat(solid) msize(vtiny) mcolor(maroon) clcolor(maroon) clwidth(thick) mlabel(timecostlab) mlabsize(small) mlabposition(5) mlabcolor(maroon))
			 (connected diffN`tr'no pwages_noopt, yaxis(2) clpat(`linetype') msize(tiny) mcolor(ltblue) clcolor(ltblue) clwidth(medthick) msymbol(none) mlabel(diffMTRnolab) mlabsize(small) mlabposition(10) mlabcolor(ltblue))
			 if g==2 & pwages_noopt < `topn' , xlabel(`grange') ylabel(0(5)25) ylabel(0(5)25, axis(2))
			 title("Financial Benefit of Reoptimizing: Percent of Base Year Income") xtitle("", size(minuscule) margin(tiny)) ytitle("Percent", axis(1)) ytitle("% Change in (1-`tr')", axis(2)) graphregion(fcolor(white))  ;
			#delimit cr                          
			graph save percent.gph, replace
			graph save output_figures/`prefix'-percent-`ma'.gph, asis replace
			save output_data/`prefix'-percent-`ma'.dta, replace   
		}
		
		
		
		* Graph how time cost changes with income, for those in relevant range FOR EACH g
		#delimit;
		twoway 
		(connected timecost pwages_noopt, clpat(solid) msize(vtiny) mcolor(maroon) clcolor(maroon) clwidth(thick) mlabel(timecostlab) mlabsize(small) mlabposition(5) mlabcolor(maroon))
		(connected diffN`tr'no pwages_noopt, yaxis(2) clpat(`linetype') msize(tiny) mcolor(ltblue) clcolor(ltblue) clwidth(medthick) msymbol(none) mlabel(diffMTRnolab) mlabsize(small) mlabposition(10) mlabcolor(ltblue))
		if pwages_noopt < `topn', 
		by(g, graphregion(fcolor(white)) plotregion(fcolor(white))  title("`tit': Time Benefit of Reoptimizing Along `ma' Margin for Two Elasticities", size(medium)) note("Note: `not'"))
		xlabel(`grange')  xtitle("`pot' `postperiod' Pre-Tax Income Without Optimizing, $'000's")
		ytitle("Hours", axis(1)) ytitle("`tr'", axis(2)) graphregion(fcolor(white))  ;
		#delimit cr                          
		graph save output_figures/`prefix'-time-elasticities-`ma'.gph, asis replace     	

		* Add labels for hours
		gen diffhourslab = ""
		
		
		* Graph change in income required to reoptimize
		#delimit;
		twoway 
		(connected diffincome pwages_noopt, clpat(solid) msize(vtiny) mcolor(maroon) clcolor(maroon) clwidth(thick) mlabel(diffhourslab) mlabsize(small) mlabposition(12) mlabcolor(maroon))
		(connected diffN`tr'no pwages_noopt, yaxis(2) clpat(`linetype') msize(tiny) mcolor(ltblue) clcolor(ltblue) clwidth(medthick) msymbol(none) mlabel(diffMTRnolab) mlabsize(small) mlabposition(5) mlabcolor(ltblue))
		if g==2 & pwages_noopt < `topn' , xlabel(`grange') title(Change in Income When Reoptimizing) 
		ytitle("Income", axis(1)) ytitle("% Change in (1-`tr')", axis(2)) xtitle("", size(minuscule) margin(tiny) ) graphregion(fcolor(white)) ;
		#delimit cr
		graph save income.gph, replace
		graph save output_figures/`prefix'-income-`ma'.gph, asis replace
		save output_data/`prefix'-income-`ma'.dta, replace
		
		* Graph change in tax rate for non-optimizers
		if "`tr'"=="MTR" local titl "Change in Marginal Tax Rates"
		else local titl "Change in Average Tax Rates"
		#delimit;
		twoway
		(connected `tr'_post_noopt pwages_noopt, msize(tiny) clpat(solid) msymbol(none) mlabel(MTR_post_nooptlab) mlabsize(small) mlabposition(11)) 
		(connected `tr'_pre pwages_noopt, msize(tiny) clpat(vshortdash) msymbol(none) mlabel(prmtrlab) mlabsize(small) mlabposition(7)) 
		(connected diffN`tr'no pwages_noopt, yaxis(2) clpat(`linetype') msize(tiny) mcolor(midblue) clcolor(midblue) clwidth(medthick) msymbol(none) mlabel(diffMTRnolab) mlabsize(small) mlabposition(7) mlabcolor(ltblue) )
		if pwages_noopt < `topn' & g == 2, ytitle("`tr' (%)", axis(1))  xlabel(`grange') xtitle("", size(minuscule) margin(tiny)) graphregion(fcolor(white))
		title("`titl'") ;
		#delimit cr    
		graph save change-noopt.gph, replace
		graph save output_figures/`prefix'-taxrate-`ma'.gph, asis replace

		* Combine graphs
		#delimit;  
		graph combine change-noopt.gph util-low.gph percent.gph income.gph, cols(2) 
		iscale(* .55) title("`tit': `ma' Margin") 
		note("`pot' `postperiod' Pre-Tax Income Without Reoptimizing, $'000's", size(vsmall) position(6) justification(center) )
		caption("Note: `not', g = 1", size(vsmall)) 
		ysize(5.8) xsize(6.4) graphregion(fcolor(white));
		#delimit cr
		graph save output_figures/`prefix'-all-`ma'.gph, asis replace
		
		erase change-noopt.gph 
		erase util-low.gph 
		erase percent.gph 
		erase income.gph
			
		*capture erase taxsim_out.dta
	}
}

*******************************************
* BUNCHING CODE
*********************************************8

* Pick the tax changes for which we want data on bunching
local bunch "tra86 ZER2006"
			
foreach bu of local bunch {
	use output_data/`bu'-taxsched, clear

	gen countMTR = 0
	* Note the kinks
	replace countMTR = MTR_post_noopt[_n-1] if MTR_post_noopt != MTR_post_noopt[_n-1]
	* Give each MTR segment a number
	gen counter = 0
	replace counter = 1 if countMTR != 0
	gen summer = sum(counter)
	
	gen countATR = 0
	replace countATR = ATR_post_noopt[_n-1] if ATR_post_noopt !=ATR_post_noopt[_n-1]
	
	* Give all taxpayers in each MTR segment a counterfactual tax rate
	bys summer: gen counterMTR = countMTR[1]
	bys summer: gen counterATR = countATR[1]
	
	* Record where the kink is as the first datapoint at the higher tax rate
	bys summer: gen kinkbegin = postpwages[1]

	* note that this generates missing values for first segment
	
	* For those who could bunch at a kink, calculate optimal location (e=.5)
	* censor below at the kink; censor above at current location; otherwise code is very similar
	* it will be faster to solve analytically : choose max(kink, analytical solution)
	
	* Solve analytically for m with counter-factual tax
	local g = 2
	gen m`g' = postpwages/(1-counterMTR)^(1/`g')

	
	* Plug in real tax to find optimal behavior
	gen dreamincome = m`g'*(1-MTR_post_noopt)^(1/`g')
	gen kink = 0
	replace kink = 1 if dreamincome <= kinkbegin
	gen optincome = dreamincome
	replace optincome = kinkbegin if kink == 1 
	
	* Find utiilty Benefit of Reoptimizing; cannot solve for optimized v. non-optimized utilities without knowing earnings
	* However, the difference is easy to know
	gen dollargain = (1-MTR_post_noopt)*(postpwages - optincome) - m`g'/(`g'+1)*((postpwages/m`g')^(`g'+1)-(optincome/m`g')^(`g'+1))
	
	* Find utiilty benefit of kinking; cannot solve for optimized v. non-optimized utilities without knowing earnings
	* However, the difference is easy to know
	gen kinkbenefit  = (1-MTR_post_noopt)*(kinkbegin - postpwages) - m`g'/(`g'+1)*((kinkbegin/m`g')^(`g'+1)-(postpwages/m`g')^(`g'+1))
	

	* Find average benefit of kinking
	gen kinkers = kinkbenefit * kink
	bys summer: egen totbenefit = mean(kinkers)
	bys summer: egen meanbenefit = mean(kink)
	bys summer: gen avebenefit = totbenefit/meanbenefit
	bys summer: egen maxbenefit = max(kinkers)
	
	replace avebenefit = int(avebenefit * 100)/100

	* Keep only the convex kinks
	replace kinkbenefit = .  if counterMTR > MTR_post_noopt
	* The first segment will not have a counter-factual: drop them
	replace kinkbenefit = . if counterMTR == .

	sort postpwages
	
	* Figure out where to cut off the x axis
	* First find highest income in a tax bracket with a kink
	preserve
	keep if kink == 1
	egen maxkinked = max(postpwages)
	local maxk = maxkinked[1]
	restore
	* Second, find lowest income in highest bracket beginning with a convex kink
	preserve
	keep if kink == 1
	bys summer: keep if _n==1
	local minmaxkinked = postpwages[_N]
	restore
	* Finally, choose cut-off for x axis as the lower of the highest kinked point and the bottom of top
	* kinked bracket + 40,000
	gen maxkinked = `maxk'
	gen minmaxkinked = `minmaxkinked'
	gen maxnum =0
	if maxkinked < minmaxkinked + 40000 replace maxnum = maxkinked
	else replace maxnum = minmaxkinked + 40000
	local maxnum = maxnum[1]
	
	* Generate a percentage benefits
	gen percentbenefits = kinkbenefit/(optincome*(1-countATR))*100* kink
	* Generate average percentage benefits				
	bys summer: egen meanpercentall = mean(percentbenefits)
	bys summer: gen meanpercent = meanpercentall/meanbenefit
	replace meanpercent = int(meanpercent*100)/100
	
	* Divide postpwages by 1000 to make better graphs
	replace pwages_noopt = postpwages/1000
	local topn = `maxnum'/1000		
	sort pwages_noopt
	
	* Add labels for percent financial cost and financial cost
	* gen percentlab = .
	gen dol = "$"
	gen per = "%"
	gen numone = .
	bys summer: replace numone = 1 if _n==1
	gen meanpercentone = meanpercent * numone
	tostring meanpercentone, replace force
	*egen percentlab = concat( meanpercentone per)
	gen percentlab = ""
	replace percentlab = meanpercentone if meanpercentone != "."
	* bys summer: replace percentlab = meanpercent if _n==1
	* gen finlab = .
	gen avebenefitone = avebenefit*numone
	tostring avebenefitone, replace force
	* egen finlab = concat(dol avebenefitone)
	gen finlab = ""
	replace finlab = avebenefitone if avebenefitone !="."
	
	* bys summer: replace finlab = avebenefit if _n==1
	 
	display "topn `topn'"
	
	capture graph set print logo off
	
	* Graph how financial cost as percentage of income changes with income, for those in relevant range
	#delimit;
	twoway (connected MTR_post_noopt pwages_noopt, clpat(dash) msize(small) msymbol(none) mcolor(maroon) clcolor(maroon) clwidth(thick) mlabel(percentlab) mlabsize(small) mlabposition(5) mlabcolor(maroon))
	 if pwages_noopt < `topn', 
	 title("Average Benefit of Bunching: Percent of Income") xtitle("Income without Optimizing") ytitle("MTR") graphregion(fcolor(white))  legend(off);
	#delimit cr                          
	* graph save percent.gph, replace
	graph save output_figures/bunch-`bu'-percent.gph, asis replace   
	graph save output_figures/bunch-`bu'-percent1.gph, replace  
	save output_data/bunch-`bu'-percent.dta, replace
	
	* Graph how financial cost changes with income, for those in relevant range
	#delimit;
	twoway (connected MTR_post_noopt pwages_noopt, clpat(dash) msize(small) msymbol(none) mcolor(maroon) clcolor(maroon) clwidth(thin) mlabel(fin) mlabsize(small) mlabposition(5) mlabcolor(maroon))
	 if pwages_noopt < `topn' , 
	 title("Average Financial Benefit of Bunching") xtitle("Income without Optimizing") ytitle("MTR") graphregion(fcolor(white))  legend(off);
	#delimit cr                          
	* graph save percent, replace
	graph save output_figures/bunch-`bu'-financial.gph, asis replace
	save output_data/bunch-`bu'-financial.dta, replace   
}

capture erase inputgraphs.dta
capture erase taxsim_out.dta

log close

*End of graphs
